@@ -44,11 +44,13 @@ void reset_signals(void)
   sigprocmask(SIG_UNBLOCK, &allsignals, NULL);
 }
 
+#define ERRBUFSIZ 2047		/* Max length of error string excl \0 */
+
 int spawnv(int logpri, char *prog, char **argv) {
   pid_t f;
   int status, pipefd[2];
-  FILE *errs;
-  char errbuf[1024], *p;
+  char errbuf[ERRBUFSIZ+1], *p, *sp;
+  int errp, errn;
   sigset_t allsignals, tmpsig, oldsig;
   
   sigfillset(&allsignals);
@@ -85,15 +87,35 @@ int spawnv(int logpri, char *prog, char **argv) {
       return -1;
     }
 
-    if ( (errs = fdopen(pipefd[0], "r")) ) {
-      while ( fgets(errbuf, 1024, errs) ) {
-	if ( (p = strchr(errbuf, '\n')) ) *p = '\0';
-	if ( errbuf[0] )	/* Don't output empty lines */
+    errp = 0;
+    do {
+      while ( (errn = read(pipefd[0], errbuf+errp, ERRBUFSIZ-errp)) == -1
+	      && errno == EINTR );
+      
+      if ( errn > 0 ) {
+	errp += errn;
+
+	sp = errbuf;
+	while ( errp && (p = memchr(sp, '\n', errp)) ) {
+	  *p++ = '\0';
+	  if ( sp[0] )	/* Don't output empty lines */
+	    syslog(logpri, ">> %s", sp);
+	  errp -= (p-sp);
+	  sp = p;
+	}
+
+	if ( errp && sp != errbuf )
+	  memmove(errbuf, sp, errp);
+
+	if ( errp >= ERRBUFSIZ ) {
+	  /* Line too long, split */
+	  errbuf[errp] = '\0';
 	  syslog(logpri, ">> %s", errbuf);
+	  errp = 0;
+	}
       }
-      fclose(errs);
-    } else
-      close(pipefd[0]);
+    } while ( errn > 0 );
+    close(pipefd[0]);
 
     if ( waitpid(f, &status, 0) != f )
       status = -1;		/* waitpid() failed */
